/* -------------------------------------------------------------------- */ 
/* SMS Client, send messages to mobile phones and pagers		*/ 
/*									*/ 
/* mtn.c 								*/
/*									*/ 
/*  Copyright (C) 1998 Alf Stockton					*/ 
/*									*/ 
/*  This library is free software; you can redistribute it and/or	*/ 
/*  modify it under the terms of the GNU Library General Public 	*/ 
/*  License as published by the Free Software Foundation; either	*/ 
/*  version 2 of the License, or (at your option) any later version.	*/ 
/*									*/ 
/*  This library is distributed in the hope that it will be useful,	*/ 
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/ 
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/ 
/*  Library General Public License for more details.			*/ 
/*									*/ 
/*  You should have received a copy of the GNU Library General Public	*/ 
/*  License along with this library; if not, write to the Free		*/ 
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/ 
/*									*/ 
/*  You can contact the author at this e-mail address:			*/ 
/*									*/ 
/*  stockton@fast.co.za 						*/ 
/*									*/ 
/* -------------------------------------------------------------------- */ 
/* $Id: mtn.c,v 3.1 1997/12/14 01:43:52 root Exp $ 
   -------------------------------------------------------------------- */ 
 
#include <stdio.h> 
#include <termios.h> 
#include <strings.h> 
 
#include <dial/modems.h> 
#include <unistd.h>
 
#include "logfile.h" 
#include "driver.h" 
#include "expect.h" 
#include "sms_modem.h" 
#include "sms_error.h" 
#include "sms_list.h" 
#include "parserc.h" 

/* -------------------------------------------------------------------- */

static char ACK1[] = "Number : 083";
static char ACK3[] = "Please enter message :";
static char ACK4[] = "Wait for confirmation ? (Y/N)";

/* -------------------------------------------------------------------- */
 
static int	fd, 
		fd_in, 
		fd_out; 
 
 
#define LONGTIMEOUT	60 
#define MAX_BUFSIZE	1024 
#define TIMEOUT 	15 
 
/* -------------------------------------------------------------------- */ 
 
static int MTN_dial(char *number, char *service); 
static void MTN_hangup(void); 
static int MTN_sendmessage(char *msisdn, char *message); 
 
/* -------------------------------------------------------------------- */ 
/* -------------------------------------------------------------------- */ 
static int MTN_dial(char *number, char *service) 
{ 
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number); 
 
	fd = SMS_dial(number, service); 
	if (fd < 0) 
	{ 
		mdmperror("Dialing"); 
		return EDIAL; 
	} 
 
	fd_in  = fd; 
	fd_out = fd; 
 
	lprintf(LOG_STANDARD, "Connection Established.\n"); 
 
	return 0; 
} 
 
 
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void MTN_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");
	hangup(fd);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int MTN_sendmessage(char *msisdn, char *message)
{
	char 	buf[8096];
	char 	nmessage[5120]; 

	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "MTN Service Login\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No MTN Service Response\n");
		MTN_hangup();
		return EMTN_NORESPONSE;
	}

	twrite(fd_out, msisdn, strlen(msisdn), TIMEOUT);
	twrite(fd_out, "\n", strlen("\n"), TIMEOUT); 

	if (expstr(fd_in, buf, msisdn, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Number Returned\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Number Returned\n");
		MTN_hangup();
		return EMTN_NONUMBER;
	}

	if (expstr(fd_in, buf, ACK3, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Message Request Received \n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Request\n");
		MTN_hangup();
		return EMTN_NONUMBER;
	}

	sleep(2);
	strcpy(nmessage, message);
	strcat(nmessage, "\n");
	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT); 

	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Wait for confirmation Received \n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Wait Confirmation Response\n");
		MTN_hangup();
		return EMTN_NODELIVERY;
	}
   
	twrite(fd_out, "n\n", strlen("n\n"), TIMEOUT);

	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "EXIT to exit Received \n");
	}
	else
	{	lprintf(LOG_STANDARD, "No EXIT to exit Response\n");
		MTN_hangup();
		return EMTN_NODELIVERY; 
	}

	twrite(fd_out, "exit\n", strlen("exit\n"), TIMEOUT); 
 
	return 0; 
} 
 
/* -------------------------------------------------------------------- */ 
/* -------------------------------------------------------------------- */ 
void mtn_deliver(SMS_list *node, char *message) 
{ 
	char    *msisdn, 
		*smsc_number, 
		*name, 
		*mservice; 
 
	int     error; 
 
	msisdn	    = get_number(node); 
	name	    = get_name(node); 
	mservice    = get_service(node); 
	smsc_number = get_smscnumber(mservice); 
 
	lprintf(LOG_VERBOSE, "Name	   :%s\n", name); 
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn); 
 
	error = MTN_dial(smsc_number, mservice); 
	if (error) 
	{ 
		set_delivery(node, error); 
		return; 
	} 
 
	error = MTN_sendmessage(msisdn, message); 
	if (error) 
	{ 
		set_delivery(node, error); 
		return; 
	} 
 
	MTN_hangup(); 
} 
 
/* -------------------------------------------------------------------- */ 
/* -------------------------------------------------------------------- */ 
SMS_list *mtn_main(SMS_list *list, char *message) 
{ 
	SMS_list 
		*node; 
 
	lprintf(LOG_VERBOSE, "%s driver called\n", mtn_device.name); 
 
	node = get_first(list); 
	while (node != NULL) 
	{ 
		set_delivery(node, 0); 
		mtn_deliver(node, message); 
		node = get_next(node); 
	} 
 
	return NULL; 
} 
 
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int mtn_validate_id(char *id)
{
	return is_numeric(id);
}

/* -------------------------------------------------------------------- */ 
/* -------------------------------------------------------------------- */ 
DEVICE_ENTRY mtn_device = {
	"MTN",
	mtn_main,
	mtn_validate_id
};

