/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* one2one.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: one2one.c,v 5.1 1998/02/01 07:10:39 root Exp root $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <termios.h>
#include <strings.h>

#include <dial/modems.h>

#include "logfile.h"
#include "driver.h"
#include "expect.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */

#define FALSE 	0
#define TRUE 	(!FALSE)

/* -------------------------------------------------------------------- */


static char ACK1[] = "ID=";
static char ACK2[] = "Welcome to the One 2 One Short Message Service\r\n";
static char ACK3[] = "Password: ";

static char ACK4[] = "Please enter below the message you wish to send\r\n";
static char ACK5[] = "Enter the One 2 One phone number to receive your message: ";
static char ACK6[] = "Goodbye (you should hangup now)";

/* -------------------------------------------------------------------- */

static int 	fd,
		fd_in,
		fd_out;

#define LONGTIMEOUT 	30
#define MAX_BUFSIZE	1024
#define TIMEOUT 	10

/* -------------------------------------------------------------------- */

static int  ONE2ONE_dial(char *number, char *service);
static void ONE2ONE_hangup(void);
static int ONE2ONE_login(void);
static int ONE2ONE_sendmessage(char *msisdn, char *message);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int ONE2ONE_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);
	
	fd = SMS_dial(number, service);
	if (fd < 0)
	{ 	lprintf(LOG_WARNING, "Failed to connect\n"); 
		return EDIAL;
	}

	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int ONE2ONE_login(void)
{
	char 	buf[1024];

	twrite(fd_out, "\r", strlen("\r"), TIMEOUT);

	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received one2one ID Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No one2one ID Request\n");
		
		ONE2ONE_hangup();
		return EONE2ONE_NORESPONSE;
	}

	twrite(fd_out, "A\r", strlen("A\r"), TIMEOUT);

	if (expstr(fd_in, buf, ACK2, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received one2one Welcome\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No one2one Welcome\n");
		
		ONE2ONE_hangup();
		return EONE2ONE_NORESPONSE;
	}

	if (expstr(fd_in, buf, ACK3, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received one2one Password Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No one2one Password Request\n");
		
		ONE2ONE_hangup();
		return EONE2ONE_NORESPONSE;
	}

	twrite(fd_out, "\r", strlen("\r"), TIMEOUT);

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void ONE2ONE_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");
	hangup(fd);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int ONE2ONE_sendmessage(char *msisdn, char *message)
{
	char 	buf[1024],
		nmessage[1024],
		nmsisdn[1024];


	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received one2one Message Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No one2one Message Request\n");
		
		ONE2ONE_hangup();
		return EONE2ONE_NONUMBER;
	}


	strcpy(nmessage, message);
	strcat(nmessage, "\r");

	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);

	if (expstr(fd_in, buf, ACK5, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Number Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Number Request\n");
		
		ONE2ONE_hangup();
		return EONE2ONE_NOMESSAGE;
	}

	strcpy(nmsisdn, msisdn);
	strcat(nmsisdn, "\r");

	twrite(fd_out, nmsisdn, strlen(nmsisdn), TIMEOUT);

	if (expstr(fd_in, buf, ACK6, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Delivery Response\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Delivery Response\n");
		
		ONE2ONE_hangup();
		return EONE2ONE_NODELIVERY;
	}

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void one2one_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int error;


	msisdn = get_number(node);
	name = get_name(node);
	mservice = get_service(node);
	smsc_number = get_smscnumber(mservice);

	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


	error = ONE2ONE_dial(smsc_number, mservice);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = ONE2ONE_login();
	if (error)
	{
		ONE2ONE_hangup();

		set_delivery(node, error);
		return;
	}

	error = ONE2ONE_sendmessage(msisdn, message);
	if (error)
	{
		ONE2ONE_hangup();

		set_delivery(node, error);
		return;
	}
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *one2one_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;


	lprintf(LOG_VERBOSE, "%s driver called\n", one2one_device.name);
	

	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		one2one_deliver(node, message);
		node = get_next(node);
	}

	ONE2ONE_hangup();

	return NULL; 
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int one2one_validate_id(char *id)
{
	return TRUE;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY one2one_device = {
	"ONE2ONE",
	one2one_main,
	one2one_validate_id
};

