/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* orange.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: orange.c,v 4.3 1998/01/17 22:38:26 root Exp $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <strings.h>

#include <dial/modems.h>

#include "logfile.h"
#include "driver.h"
#include "expect.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */

#if 0
static char ACK1[] = "Welcome to Orange Dial-up Messaging\r\n";
#endif

static char ACK2[] = "Please choose an option (do not press return) : \r\n"
		     "\r\n"	
		     "       S   :- Send a message\r\n"
		     "       H   :- Help\r\n"
		     "       E   :- Exit\r\n";

static char ACK3[] = "Enter destination Orange or Hutchison Pager number and press return\r\n"
		     "\r\n"
		     ":";

static char ACK4[] = "Type your message (160 characters max) and press return to send\r\n"
		     "\r\n"
		     ":";

static char ACK5[] = "Message accepted - thankyou\r\n"
		     "\r\n"
		     "Please choose an option (do not press return) : \r\n"
		     "\r\n"
		     "       S   :- Send a message\r\n"
		     "       H   :- Help\r\n"
		     "       E   :- Exit\r\n";

static char ACK6[] = "Thanks for calling.\r\n"
		     "\r\n"
		     "The future's bright. The future's Orange.";

/* -------------------------------------------------------------------- */

static int 	fd,
		fd_in,
		fd_out;

#define LONGTIMEOUT 	30
#define MAX_BUFSIZE	1024
#define TIMEOUT 	5

/* -------------------------------------------------------------------- */

static int connection_status;

/* -------------------------------------------------------------------- */

#define ORANGE_NOT_CONNECTED 	0
#define ORANGE_CONNECTED 	1

/* -------------------------------------------------------------------- */

static int ORANGE_dial(char *number, char *service);
static void ORANGE_hangup(void);
static int ORANGE_sendmessage(char *msisdn, char *message);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int ORANGE_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);
	
	fd = SMS_dial(number, service);
	if (fd < 0)
	{
		mdmperror("Dialing");
		return EDIAL;
	}

	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void ORANGE_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");	
	hangup(fd);

	connection_status = ORANGE_NOT_CONNECTED;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int ORANGE_sendmessage(char *msisdn, char *message)
{
	char buf[1024];
	char nmessage[1024];


#if 0
	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Orange Service Login\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Orange Service Response\n");
		
		ORANGE_hangup();
		return EORANGE_NORESPONSE;
	}
#endif


	twrite(fd_out, "H", strlen("H"), TIMEOUT);


	if (expstr(fd_in, buf, ACK2, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Orange Service Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Orange Service Request\n");
		
		ORANGE_hangup();
		return EORANGE_NOSERVICE;
	}



	twrite(fd_out, "S", strlen("S"), TIMEOUT);


	if (expstr(fd_in, buf, ACK3, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Number Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Number Request\n");
		
		ORANGE_hangup();
		return EORANGE_NONUMBER;
	}



	twrite(fd_out, msisdn, strlen(msisdn), TIMEOUT);
	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);


	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Request\n");
		
		ORANGE_hangup();
		return EORANGE_NOMESSAGE;
	}


	strcpy(nmessage, message);
	strcat(nmessage, "\r\n");

	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);

	if (expstr(fd_in, buf, ACK5, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Delivery Response\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Delivery Response\n");
		
		ORANGE_hangup();
		return EORANGE_NODELIVERY;
	}


	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int ORANGE_send_disconnect(void)
{
	char buf[1024];


	twrite(fd_out, "E", strlen("E"), TIMEOUT);

	if (expstr(fd_in, buf, ACK6, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Disconnection Response\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Disconnection Response\n");
		
		ORANGE_hangup();
		return EORANGE_NODISCONNECT;
	}

	return 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void orange_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int 	error;
	


	msisdn      = get_number(node);
	name        = get_name(node);
	mservice    = get_service(node);
	smsc_number = get_smscnumber(mservice);

	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


	if (connection_status == ORANGE_NOT_CONNECTED)
	{
	 	error = ORANGE_dial(smsc_number, mservice);
		if (error)
		{
			set_delivery(node, error);
			return;
		}

		connection_status = ORANGE_CONNECTED;
	}

	error = ORANGE_sendmessage(msisdn, message);
	if (error)
	{
		ORANGE_hangup();

		set_delivery(node, error);
		return;
	}
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *orange_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;


	lprintf(LOG_VERBOSE, "%s driver called\n", orange_device.name);

	connection_status = ORANGE_NOT_CONNECTED;
	
	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		orange_deliver(node, message);
		node = get_next(node);
	}

	ORANGE_send_disconnect();
	ORANGE_hangup();

	return NULL;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int orange_validate_id(char *id)
{
	return is_numeric(id);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY orange_device = {
	"ORANGE",
	orange_main,
	orange_validate_id
};


