/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* pageone.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: pageone.c,v 4.3 1998/01/17 22:37:40 root Exp $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <termios.h>
#include <strings.h>

#include <dial/modems.h>

#include "logfile.h"
#include "driver.h"
#include "expect.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */

#define FALSE 	0
#define TRUE 	(!FALSE)

/* -------------------------------------------------------------------- */


static char ACK1[] = "ID=\r\n";
static char ACK2[] = "ENTER SUBSCRIBER NUMBER\n\r";
static char ACK4[] = "ENTER MESSAGE: ";
static char ACK5[] = "PAGE ACCEPTED\n\r";

/* -------------------------------------------------------------------- */

static int 	fd,
		fd_in,
		fd_out;

#define LONGTIMEOUT 	30
#define MAX_BUFSIZE	1024
#define TIMEOUT 	10

/* -------------------------------------------------------------------- */

static int connection_status;

/* -------------------------------------------------------------------- */

#define PAGEONE_NOT_CONNECTED 	0
#define PAGEONE_CONNECTED 	1

/* -------------------------------------------------------------------- */

static int  PAGEONE_dial(char *number, char *service);
static void PAGEONE_hangup(void);
static int PAGEONE_login(void);
static int PAGEONE_sendmessage(char *msisdn, char *message);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int PAGEONE_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);
	
	fd = SMS_dial(number, service);
	if (fd < 0)
	{
		mdmperror("Dialing");
		return EDIAL;
	}

	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int PAGEONE_login(void)
{
	char 	buf[1024];


	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Pageone ID Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Pageone ID Request\n");
		
		PAGEONE_hangup();
		return EPAGEONE_NORESPONSE;
	}

	twrite(fd_out, "PG1\r", strlen("PG1\r"), TIMEOUT);

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void PAGEONE_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");
	hangup(fd);

	connection_status = PAGEONE_NOT_CONNECTED;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int PAGEONE_sendmessage(char *msisdn, char *message)
{
	char 	buf[1024],
		nmessage[1024],
		nmsisdn[1024];


	if (expstr(fd_in, buf, ACK2, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Pageone Number Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Pageone Number Request\n");
		
		PAGEONE_hangup();
		return EPAGEONE_NONUMBER;
	}

	strcpy(nmsisdn, msisdn);
	strcat(nmsisdn, "\r");

	twrite(fd_out, nmsisdn, strlen(nmsisdn), TIMEOUT);

	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Request\n");
		
		PAGEONE_hangup();
		return EPAGEONE_NOMESSAGE;
	}


	strcpy(nmessage, message);
	strcat(nmessage, "\r");

	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);

	if (expstr(fd_in, buf, ACK5, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Delivery Response\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Delivery Response\n");
		
		PAGEONE_hangup();
		return EPAGEONE_NODELIVERY;
	}

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void pageone_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int error;


	msisdn = get_number(node);
	name = get_name(node);
	mservice = get_service(node);
	smsc_number = get_smscnumber(mservice);

	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


	if (connection_status == PAGEONE_NOT_CONNECTED)
	{
		error = PAGEONE_dial(smsc_number, mservice);
		if (error)
		{
			set_delivery(node, error);
			return;
		}

 		connection_status = PAGEONE_CONNECTED;

		error = PAGEONE_login();
		if (error)
		{
			PAGEONE_hangup();

			set_delivery(node, error);
			return;
		}
	}

	error = PAGEONE_sendmessage(msisdn, message);
	if (error)
	{
		PAGEONE_hangup();

		set_delivery(node, error);
		return;
	}
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *pageone_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;


	lprintf(LOG_VERBOSE, "%s driver called\n", pageone_device.name);
	
	connection_status = PAGEONE_NOT_CONNECTED;

	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		pageone_deliver(node, message);
		node = get_next(node);
	}

	PAGEONE_hangup();

	return NULL; 
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int pageone_validate_id(char *id)
{
	return TRUE;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY pageone_device = {
	"PAGEONE",
	pageone_main,
	pageone_validate_id
};

