/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* sms_client.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: sms_client.c,v 4.3 1998/01/17 22:49:15 root Exp root $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>

#include <dial/modems.h>

#include "sms_error.h"
#include "sms_ascii.h"
#include "sms_list.h"
#include "expect.h"
#include "logfile.h"
#include "parserc.h"
#include "driver.h"

/* -------------------------------------------------------------------- */

#if defined(MVERSION)
#define VERSION	MVERSION
#else
#define VERSION	"Version Unknown"
#endif

/* -------------------------------------------------------------------- */

#define FALSE 		0
#define TRUE 		(!FALSE)

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */

#define MAX_TAPMESSAGE	150
#define MAX_MESSAGE	512

/* -------------------------------------------------------------------- */

void get_message(char *message);
void usage(char *file);
int main(int, char *[]);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void get_message(char *message)
{
	int	i,
		c;

		
	if (isatty(fileno(stdin)))
	{	printf("Enter your message and end with Control-D\n");
	}
		
	for (i=0; i<MAX_TAPMESSAGE; i++)
	{
		c = getchar();
		if (c == EOF)
		{	break;
		}

		message[i] = c;
	}

	if (i == MAX_TAPMESSAGE)
	{
		lprintf(LOG_ERROR, "Message too long\n");
		exit(EMESSAGETOOLONG);
	}

	message[i] = '\0';
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void usage(char *file)
{
	char 	blank[128], 
		*src, 
		*dst;


	lprintf(LOG_STANDARD, "Usage: %s [-v]\n", file);
	lprintf(LOG_STANDARD, "Usage: %s [-d]\n", file);
	lprintf(LOG_STANDARD, "       %s [-q][-l loglevel]\n", file);


	src = file;
	dst = blank;
	while(*src != '\0')
	{
		*dst = ' ';
		
		dst++;
		src++;
	}

	*dst = '\0';

	lprintf(LOG_STANDARD, "       %s [service:]number|name[,name|[service:]number...]\n", blank);
	lprintf(LOG_STANDARD, "       %s [message]\n", blank);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int main(int argc, char *argv[])
{
	DEVICE_ENTRY
		*device;

	char 	*mservice,
		message[MAX_MESSAGE],
		*ptr,
		*sms_default_service = NULL,
		*protocol;
		

	SMS_parent_list 
		*parent_node,
		*parent_list;

	SMS_list
		*list,
		*numbers,
		*delivered;
		
		
	FILE	*fp[2];
		
	int	num_args,
		nind,
		mind,
		c,
		delivery_error = 0;

	time_t	start_time,
		end_time;
		
		
	/* ---------------------------- */

	while ((c = getopt (argc, argv, "vql:m:d")) != -1)
        {
                switch (c)
                {
                        case 'q':
				console_log = FALSE;
                                break;

                        case 'd':
				display_drivers();
				exit(0);

                        case 'v':
				lprintf(LOG_STANDARD, "sms_client %s\n", VERSION);
                                exit(0);

                        case 'l':  
				current_loglevel = (int)strtol(optarg, &ptr, 10);
				if (ptr == optarg)
				{
					current_loglevel = LOG_STANDARD;
					lprintf(LOG_ERROR, "Option l requires an argument\n");

	     	                        usage(argv[0]);
                	                exit(EUSAGE);
				}
                                
                                break;

                        case '?':
                                lprintf(LOG_ERROR, "Unknown option `-%c'\n", optopt);
                                usage(argv[0]);
                                exit(EUSAGE);

                        default:
                                abort ();
                }
        }
                        

	/* ---------------------------- */

	num_args = argc - optind;

	if ((num_args < 1) || (num_args > 2))
	{
		usage(argv[0]);
		exit(EUSAGE);
	}

	nind = optind;
	mind = optind +1;

	/* ---------------------------- */
	/* Get Default SERVICE 		*/

	SMS_dual_openrc(fp);

	if (sms_default_service == NULL)
	{
		sms_default_service = SMS_dual_get_smsrc_value(fp, "SMS_default_service");
		if (sms_default_service == NULL)
		{
			lprintf(LOG_ERROR, "Could not find SMS_default_service\n");
			exit(ENOSERVICE);
		}
	}

	SMS_dual_closerc(fp);


	/* ---------------------------- */
	/* Get and expand NAMES|NUMBERS	*/
	
	SMS_dual_openrc(fp);
	numbers = SMS_expandnumber(fp, "<NULL>", argv[nind], sms_default_service);
	SMS_dual_closerc(fp);


	/* ---------------------------- */
	/* Check NAMES|NUMBERS		*/

	if (validate_expanded_numbers(numbers))
	{
		lprintf(LOG_ERROR, "Expanding names\n"); 
		exit(ENAMEEXPANSION);
	}

	/* ---------------------------- */


	if (num_args == 2)
	{
		if (strlen(argv[mind]) > MAX_TAPMESSAGE)
		{
			lprintf(LOG_ERROR, "Message too long\n");
			exit(EMESSAGETOOLONG);
		}

		strncpy(message, argv[mind], MAX_MESSAGE);
	}
	else
	{	get_message(message);
	}


	/* ---------------------------- */

	time(&start_time);


	parent_list = gather(numbers);
	parent_node = get_first_parent(parent_list);
	while (parent_node != NULL)
	{
		list = get_child(parent_node);		

		mservice = get_service(list);
		protocol = get_protocol(mservice);

		device = get_device(protocol);
		if (device == NULL)
		{	lprintf(LOG_ERROR, "Driver for service %s NOT found\n", mservice);
		}
		else
		{
			(*device->main)(list, message);

			delivered = list;
			while (delivered != NULL)
			{
				if (get_delivery(delivered))
				{
					lprintf(LOG_WARNING, "Could not deliver message to %s on %s delivery code %d\n", 
				        	get_name(delivered), 
					        get_number(delivered),
					        get_delivery(delivered));

					delivery_error = 1;

				}

				lprintf(LOG_STANDARD, "[%03d] %s:%s\n", 
				       get_delivery(delivered),
				       get_service(delivered),
				       get_number(delivered));

				delivered = get_next(delivered);
			}
		}
		
		parent_node = get_next_parent(parent_node);
	}


	time(&end_time);
	lprintf(LOG_STANDARD, "Total Ellapsed Time: %d Seconds\n", end_time - start_time);	

	free_list(numbers);
	return delivery_error;
}


