/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* tap.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: tap.c,v 4.3 1998/01/17 22:49:15 root Exp $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#include <dial/modems.h>

#include "expect.h"
#include "logfile.h"
#include "sms_ascii.h"
#include "driver.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */




#if 0

#define __MAX_TRIES 	 5	/* Number of Login response attempts	*/
#define __TIMEOUT 	 5	/* General timeout for reads		*/
#define __LONGTIMEOUT 	 30	/* Extra timeout for message response	*/
#define __MAX_TAPMESSAGE 150




/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int max_tries(void)
{
	static long value = -1;

	if (value == -1)
	{
		if (SMS_open_dual_getnamevalue_numeric("SMS_max_tries", &value) == -1)
		{	
			value = __MAX_TRIES;
		}
	}

	return (int)value;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int timeout(void)
{
	static long value = -1;

	if (value == -1)
	{
		if (SMS_open_dual_getnamevalue_numeric("SMS_timeout", &value) == -1)
		{	
			value = __TIMEOUT;
		}
	}

	return (int)value;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int longtimeout(void)
{
	static long value = -1;

	if (value == -1)
	{
		if (SMS_open_dual_getnamevalue_numeric("SMS_longtimeout", &value) == -1)
		{	
			value = __LONGTIMEOUT;
		}
	}

	return (int)value;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int max_tapmessage(void)
{
	static long value = -1;

	if (value == -1)
	{
		if (SMS_open_dual_getnamevalue_numeric("SMS_max_tapmessage", &value) == -1)
		{	
			value = __MAX_TAPMESSAGE;
		}
	}

	return (int)value;
}

/* -------------------------------------------------------------------- */

#define MAX_TRIES	max_tries()
#define TIMEOUT 	timeout()
#define LONGTIMEOUT 	longtimeout()
#define MAX_TAPMESSAGE	max_tapmessage()

#else

#define MAX_TRIES	5
#define TIMEOUT 	5
#define LONGTIMEOUT 	30
#define MAX_TAPMESSAGE	150

#endif


#define MAX_ENVELOPE 	64
#define MAX_NUMBER	20
#define MAX_MOBILENUM	20
#define MAX_MESSAGE	512
#define MAX_BUFSIZE	1024


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */

char TAP_charset[] =
	{
		0x20 /* 0x00    */, 0x20 /* 0x01    */, 0x20 /* 0x02    */,
		0x20 /* 0x03    */, 0x20 /* 0x04    */, 0x20 /* 0x05    */,
		0x20 /* 0x06    */, 0x20 /* 0x07    */, 0x20 /* 0x08    */,
		0x20 /* 0x09    */, 0x0A /* 0x0A LF */, 0x20 /* 0x0B    */,
		0x20 /* 0x0C    */, 0x0A /* 0x0D LF */, 0x20 /* 0x0E    */,
		0x20 /* 0x0F    */, 0x20 /* 0x10    */, 0x20 /* 0x11    */,
		0x20 /* 0x12    */, 0x20 /* 0x13    */, 0x20 /* 0x14    */,
		0x20 /* 0x15    */, 0x20 /* 0x16    */, 0x20 /* 0x17    */,
		0x20 /* 0x18    */, 0x20 /* 0x19    */, 0x20 /* 0x1A    */,
		0x20 /* 0x1B    */, 0x20 /* 0x1C    */, 0x20 /* 0x1D    */,
		0x20 /* 0x1E    */, 0x20 /* 0x1F    */, 0x20 /* 0x20    */,
		0x21 /* 0x21 !  */, 0x22 /* 0x22 "  */, 0x23 /* 0x23 #  */,
		0x24 /* 0x24 $  */, 0x25 /* 0x25 %  */, 0x26 /* 0x26 &  */,
		0x27 /* 0x27 '  */, 0x28 /* 0x28 (  */, 0x29 /* 0x29 )  */,
		0x2A /* 0x2A *  */, 0x2B /* 0x2B +  */, 0x2C /* 0x2C ,  */,
		0x2D /* 0x2D -  */, 0x2E /* 0x2E .  */, 0x2F /* 0x2F /  */,
		0x30 /* 0x30 0  */, 0x31 /* 0x31 1  */, 0x32 /* 0x32 2  */,
		0x33 /* 0x33 3  */, 0x34 /* 0x34 4  */, 0x35 /* 0x35 5  */,
		0x36 /* 0x36 6  */, 0x37 /* 0x37 7  */, 0x38 /* 0x38 8  */,
		0x39 /* 0x39 9  */, 0x3A /* 0x3A :  */, 0x3B /* 0x3B ;  */,
		0x3C /* 0x3C <  */, 0x3D /* 0x3D =  */, 0x3E /* 0x3E >  */,
		0x3F /* 0x3F ?  */, 0x40 /* 0x40 @  */, 0x41 /* 0x41 A  */,
		0x42 /* 0x42 B  */, 0x43 /* 0x43 C  */, 0x44 /* 0x44 D  */,
		0x45 /* 0x45 E  */, 0x46 /* 0x46 F  */, 0x47 /* 0x47 G  */,
		0x48 /* 0x48 H  */, 0x49 /* 0x49 I  */, 0x4A /* 0x4A J  */,
		0x4B /* 0x4B K  */, 0x4C /* 0x4C L  */, 0x4D /* 0x4D M  */,
		0x4E /* 0x4E N  */, 0x4F /* 0x4F O  */, 0x50 /* 0x50 P  */,
		0x51 /* 0x51 Q  */, 0x52 /* 0x52 R  */, 0x53 /* 0x53 S  */,
		0x54 /* 0x54 T  */, 0x55 /* 0x55 U  */, 0x56 /* 0x56 V  */,
		0x57 /* 0x57 W  */, 0x58 /* 0x58 X  */, 0x59 /* 0x59 Y  */,
		0x5A /* 0x5A Z  */, 0x20 /* 0x5B    */, 0x20 /* 0x5C    */,
		0x20 /* 0x5D    */, 0x20 /* 0x5E    */, 0x20 /* 0x5F    */,
		0x20 /* 0x60    */, 0x61 /* 0x61 a  */, 0x62 /* 0x62 b  */,
		0x63 /* 0x63 c  */, 0x64 /* 0x64 d  */, 0x65 /* 0x65 e  */,
		0x66 /* 0x66 f  */, 0x67 /* 0x67 g  */, 0x68 /* 0x68 h  */,
		0x69 /* 0x69 i  */, 0x6A /* 0x6A j  */, 0x6B /* 0x6B k  */,
		0x6C /* 0x6C l  */, 0x6D /* 0x6D m  */, 0x6E /* 0x6E n  */,
		0x6F /* 0x6F o  */, 0x70 /* 0x70 p  */, 0x71 /* 0x71 q  */,
		0x72 /* 0x72 r  */, 0x73 /* 0x73 s  */, 0x74 /* 0x74 t  */,
		0x75 /* 0x75 u  */, 0x76 /* 0x76 v  */, 0x77 /* 0x77 w  */,
		0x78 /* 0x78 x  */, 0x79 /* 0x79 y  */, 0x7A /* 0x7A z  */,
		0x20 /* 0x7B    */, 0x20 /* 0x7C    */, 0x20 /* 0x7D    */,
		0x20 /* 0x7E    */, 0x20 /* 0x7F    */, 0x20 /* 0x80    */,
		0x20 /* 0x81    */, 0x20 /* 0x82    */, 0x20 /* 0x83    */,
		0x20 /* 0x84    */, 0x20 /* 0x85    */, 0x20 /* 0x86    */,
		0x20 /* 0x87    */, 0x20 /* 0x88    */, 0x20 /* 0x89    */,
		0x20 /* 0x8A    */, 0x20 /* 0x8B    */, 0x20 /* 0x8C    */,
		0x20 /* 0x8D    */, 0x20 /* 0x8E    */, 0x20 /* 0x8F    */,
		0x20 /* 0x90    */, 0x20 /* 0x91    */, 0x20 /* 0x92    */,
		0x20 /* 0x93    */, 0x20 /* 0x94    */, 0x20 /* 0x95    */,
		0x20 /* 0x96    */, 0x20 /* 0x97    */, 0x20 /* 0x98    */,
		0x20 /* 0x99    */, 0x20 /* 0x9A    */, 0x20 /* 0x9B    */,
		0x20 /* 0x9C    */, 0x20 /* 0x9D    */, 0x20 /* 0x9E    */,
		0x20 /* 0x9F    */, 0x20 /* 0xA0    */, 0x20 /* 0xA1    */,
		0x20 /* 0xA2    */, 0x20 /* 0xA3    */, 0x20 /* 0xA4    */,
		0x20 /* 0xA5    */, 0x20 /* 0xA6    */, 0x20 /* 0xA7    */,
		0x20 /* 0xA8    */, 0x20 /* 0xA9    */, 0x20 /* 0xAA    */,
		0x20 /* 0xAB    */, 0x20 /* 0xAC    */, 0x20 /* 0xAD    */,
		0x20 /* 0xAE    */, 0x20 /* 0xAF    */, 0x20 /* 0xB0    */,
		0x20 /* 0xB1    */, 0x20 /* 0xB2    */, 0x20 /* 0xB3    */,
		0x20 /* 0xB4    */, 0x20 /* 0xB5    */, 0x20 /* 0xB6    */,
		0x20 /* 0xB7    */, 0x20 /* 0xB8    */, 0x20 /* 0xB9    */,
		0x20 /* 0xBA    */, 0x20 /* 0xBB    */, 0x20 /* 0xBC    */,
		0x20 /* 0xBD    */, 0x20 /* 0xBE    */, 0x20 /* 0xBF    */,
		0x20 /* 0xC0    */, 0x20 /* 0xC1    */, 0x20 /* 0xC2    */,
		0x20 /* 0xC3    */, 0x20 /* 0xC4    */, 0x20 /* 0xC5    */,
		0x20 /* 0xC6    */, 0x20 /* 0xC7    */, 0x20 /* 0xC8    */,
		0x20 /* 0xC9    */, 0x20 /* 0xCA    */, 0x20 /* 0xCB    */,
		0x20 /* 0xCC    */, 0x20 /* 0xCD    */, 0x20 /* 0xCE    */,
		0x20 /* 0xCF    */, 0x20 /* 0xD0    */, 0x20 /* 0xD1    */,
		0x20 /* 0xD2    */, 0x20 /* 0xD3    */, 0x20 /* 0xD4    */,
		0x20 /* 0xD5    */, 0x20 /* 0xD6    */, 0x20 /* 0xD7    */,
		0x20 /* 0xD8    */, 0x20 /* 0xD9    */, 0x20 /* 0xDA    */,
		0x20 /* 0xDB    */, 0x20 /* 0xDC    */, 0x20 /* 0xDD    */,
		0x20 /* 0xDE    */, 0x20 /* 0xDF    */, 0x20 /* 0xE0    */,
		0x20 /* 0xE1    */, 0x20 /* 0xE2    */, 0x20 /* 0xE3    */,
		0x20 /* 0xE4    */, 0x20 /* 0xE5    */, 0x20 /* 0xE6    */,
		0x20 /* 0xE7    */, 0x20 /* 0xE8    */, 0x20 /* 0xE9    */,
		0x20 /* 0xEA    */, 0x20 /* 0xEB    */, 0x20 /* 0xEC    */,
		0x20 /* 0xED    */, 0x20 /* 0xEE    */, 0x20 /* 0xEF    */,
		0x20 /* 0xF0    */, 0x20 /* 0xF1    */, 0x20 /* 0xF2    */,
		0x20 /* 0xF3    */, 0x20 /* 0xF4    */, 0x20 /* 0xF5    */,
		0x20 /* 0xF6    */, 0x20 /* 0xF7    */, 0x20 /* 0xF8    */,
		0x20 /* 0xF9    */, 0x20 /* 0xFA    */, 0x20 /* 0xFB    */,
		0x20 /* 0xFC    */, 0x20 /* 0xFD    */, 0x20 /* 0xFE    */,
		0x20 /* 0xFF    */
	};


/* -------------------------------------------------------------------- */

static char ACK1[] = { 'I', 'D', '=', '\0' };
static char ACK2[] = { S_CR, S_ACK, S_CR, '\0' };
static char ACK3[] = { S_ESC, '[',  'p',  S_CR, '\0' };
static char ACK4[] = { S_CR, '\0' };
static char ACK5[] = { S_CR, S_ACK, S_CR, '\0' };
static char ACK6[] = { S_CR, S_NAK, S_CR, '\0' };
static char ACK7[] = { S_CR, S_RS, S_CR, '\0' };
static char ACK8[] = { S_ESC, S_EOT, S_CR, '\0' };

static char REQ1[] = { S_CR, '\0' };
static char REQ2[] = { S_ESC, 'P', 'G', '1', S_CR, '\0' };
static char REQ3[] = { S_STX, '\0' };
static char REQ4[] = { S_CR, '\0' };
static char REQ5[] = { S_CR, S_ETX, '\0' };
static char REQ6[] = { S_CR, '\0' };
static char REQ7[] = { S_EOT, S_CR, '\0' };

/* -------------------------------------------------------------------- */


static int 	fd,
		fd_in,
		fd_out;

static char buf[MAX_BUFSIZE +1];

/* -------------------------------------------------------------------- */

static void TAP_checksum(char *);
static void TAP_hangup(void);
static void TAP_buildmessage(char *, char *, char *);

static int TAP_disconnect(void);
static int TAP_login(void);
static int TAP_sendmessage(char *, char *);
static int TAP_dial(char *, char *);

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void TAP_checksum(char *message)
{
	int 	i,
		checksum;

		
	checksum  = 0;
	for (i=0; i<strlen(message); i++)
	{	checksum += message[i];
	}

	checksum = (checksum & 0x0FFF);	/* Mask low 12Bits */

	message[i]   = (((checksum & 0x0F00) >> 8) | 0x30);
	message[i+1] = (((checksum & 0x00F0) >> 4) | 0x30);
	message[i+2] = (((checksum & 0x000F)     ) | 0x30);
	message[i+3] = '\0';
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void TAP_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");	
	hangup(fd);
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int TAP_disconnect(void)
{
	lprintf(LOG_STANDARD, "Disconnect...\n");
	
	twrite(fd_out, REQ7, strlen(REQ7), TIMEOUT);
	if (expnstr(fd_in, buf, ACK8, strlen(ACK8), TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Disconnected from SMSC\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "Failed to disconnect from SMSC\n");
		
		TAP_hangup();
		return ETAP_NODISCONNECT;
	}

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int TAP_login(void)
{
	int 	i;


	lprintf(LOG_STANDARD, "Login...\n");
	

	/* Resend the S_CR every 2 seconds	*/
	/* until ID= received, only try		*/
	/* 5 times then give up.		*/

	for (i=0; i < MAX_TRIES; i++)
	{
		twrite(fd_out, REQ1, strlen(REQ1), TIMEOUT);

#if 0
		if (expnstr(fd_in, buf, ACK1, strlen(ACK1), TIMEOUT) == 0)
#else
		if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
#endif
		{	
			lprintf(LOG_STANDARD, "SMSC Acknowledgment received\n");
	
			break;
		}

		lprintf(LOG_STANDARD, "SMSC Retry for Acknowledgment\n");
	
		sleep(2);
	}

	if (i == MAX_TRIES)
	{
		lprintf(LOG_STANDARD, "Failed Acknowledgment %d retries attempted\n", MAX_TRIES);
		
		TAP_hangup();
		return ETAP_NOACK;
	}

	
	/* Send S_ESC PG1 S_CR and get back	*/
	/* the acknowledgement S_CR S_ACK S_CR	*/

	twrite(fd_out, REQ2, strlen(REQ2), TIMEOUT);
	if (expnstr(fd_in, buf, ACK2, strlen(ACK2), TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Login successful\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "Login failed\n");
	
		TAP_hangup();
		return ETAP_NOLOGIN;
	}

	if (expnstr(fd_in, buf, ACK3, strlen(ACK3), TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Ready to receive message\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "SMSC NOT Ready to receive message\n");
		
		TAP_hangup();
		return ETAP_NOREADY;
	}

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void TAP_buildmessage(char *tap_message, char *msisdn, char *message)
{
	int 	i;
	char	*ptr;


	/* ---------------------------- */

	tap_message[0] = '\0';
	
	strcat(tap_message, REQ3);
	strcat(tap_message, msisdn);
	strcat(tap_message, REQ4);

	
	/* Convert message so that it contains	*/
	/* only TAP valid characters		*/
	/* any invalid characters will be	*/
	/* mapped to a SPACE			*/
	/* Add to tap_message			*/

	ptr = tap_message;
	while(*ptr != '\0')
	{	ptr++;
	}

	for (i=0; i<strlen(message) +1; i++)
	{	*ptr = TAP_charset[(int)message[i]];
		ptr++;
	}

	*ptr = '\0';

	strcat(tap_message, REQ5);


	/* Generate TAP checksum. Consisting	*/
	/* of 3 characters. Append this to end	*/
	/* of tap_message.			*/

	TAP_checksum(tap_message);

	strcat(tap_message, REQ6);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int TAP_sendmessage(char *msisdn, char *message)
{
	char 	tap_message[MAX_MESSAGE + MAX_ENVELOPE];


	TAP_buildmessage(tap_message, msisdn, message);
	
	twrite(fd_out, tap_message, strlen(tap_message), TIMEOUT);
	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, LONGTIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Response: %s\n", buf);
	}
	else
	{
		lprintf(LOG_STANDARD, "No Message Response\n");
		
		TAP_hangup();
		return ETAP_NOMESSAGE;
	}


	if (expnstr(fd_in, buf, ACK5, strlen(ACK5), TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Successful message submission\n");
	}
	else
	{ 	if (strncmp(buf, ACK6, strlen(ACK6)) == 0)
		{
			lprintf(LOG_STANDARD, "Message submission failed - Transmission incomprehensible\n");
		}
		else if (strncmp(buf, ACK7, strlen(ACK7)) == 0)
		{
			lprintf(LOG_STANDARD, "Message submission failed - Problem with message\n");
		}
		else
		{
			lprintf(LOG_STANDARD, "Message submission failed - Reason Unknown\n");
		}
		
		TAP_hangup();
		return ETAP_NODELIVERY;
	}

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int TAP_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);
	
	fd = SMS_dial(number, service);
	if (fd < 0)
	{
		mdmperror("Dialing");
		return EDIAL;
	}

	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}






/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void tap_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int 	error;
	

	msisdn = get_number(node);
	name = get_name(node);
	mservice = get_service(node);
	smsc_number = get_smscnumber(mservice);


	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


 	error = TAP_dial(smsc_number, mservice);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = TAP_login();
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = TAP_sendmessage(msisdn, message);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = TAP_disconnect();
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	TAP_hangup();
}



/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *tap_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;


	lprintf(LOG_VERBOSE, "%s driver called\n", tap_device.name);

	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		tap_deliver(node, message);
		node = get_next(node);
	}

	return NULL; 
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int tap_validate_id(char *id)
{
	return is_numeric(id);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY tap_device = {
	"TAP",
	tap_main,
	tap_validate_id
};
