/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* vodacom.c								*/
/*									*/
/*  Copyright (C) 1998 Alf Stockton                                     */ 
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: vodacom.c,v 3.1 1997/12/14 01:43:52 root Exp $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <termios.h>
#include <strings.h>

#include <dial/modems.h>

#include "logfile.h"
#include "driver.h"
#include "expect.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */

static char ACK1[] = "followed by RETURN, (or RETURN to quit)"; 
static char ACK2[] = "Please type your message, (maximum 160 characters), followed by RETURN.";

/* -------------------------------------------------------------------- */

static int 	fd,
		fd_in,
		fd_out;

		
#define LONGTIMEOUT 	45
#define MAX_BUFSIZE	1024
#define TIMEOUT 	15

/* -------------------------------------------------------------------- */

static int VODACOM_dial(char *number, char *service);
static void VODACOM_hangup(void);
static int VODACOM_sendmessage(char *msisdn, char *message);
	
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int VODACOM_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);

	fd = SMS_dial(number, service);
	if (fd < 0)
	{
		mdmperror("Dialing");
		return EDIAL;
	}
	
	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void VODACOM_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");	
	hangup(fd);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int VODACOM_sendmessage(char *msisdn, char *message)
{
	char 	buf[1024];
	char 	nmessage[1024];


	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Number Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Number Request\n");
		VODACOM_hangup();
		return EVODACOM_NORESPONSE;
	}

	sprintf(nmessage,"%s\r\n",msisdn);
	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);
   
	if (expstr(fd_in, buf, msisdn, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Number Returned\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Number Returned\n");
		VODACOM_hangup();
		return EVODACOM_NONUMBER;
	}

	/* twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT); */

	if (expstr(fd_in, buf, ACK2, MAX_BUFSIZE, LONGTIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Message Request\n");
		VODACOM_hangup();
		return EVODACOM_NONUMBER;
	}

	sprintf(nmessage,"%s\r\n",message);
	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);
	sprintf(nmessage, "\r\n\r\n>%s", message);

	if (expstr(fd_in, buf, nmessage, MAX_BUFSIZE, TIMEOUT) == 0)
        {
	        lprintf(LOG_STANDARD, "Received Message back\n");
        }
	else
        {        lprintf(LOG_STANDARD, "No Message Returned\n");
	        VODACOM_hangup();
	        return EVODACOM_NOMESSAGE;
        }
    
	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);

	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Number Request\n");
	}
	else
	{	lprintf(LOG_STANDARD, "No Number Request\n");
		VODACOM_hangup();
		return EVODACOM_NOMESSAGE;
	}

	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);

	return 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void vodacom_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int 	error;
	

	msisdn      = get_number(node);
	name        = get_name(node);
	mservice    = get_service(node);
	smsc_number = get_smscnumber(mservice);

	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


 	error = VODACOM_dial(smsc_number, mservice);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = VODACOM_sendmessage(msisdn, message);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	VODACOM_hangup();
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *vodacom_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;

	lprintf(LOG_VERBOSE, "%s driver called\n", vodacom_device.name);
	
	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		vodacom_deliver(node, message);
		node = get_next(node);
	}

	return NULL; 
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int vodacom_validate_id(char *id)
{
	return is_numeric(id);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY vodacom_device = {
	"VODACOM",
	vodacom_main,
	vodacom_validate_id
};


