/* -------------------------------------------------------------------- */
/* SMS Client, send messages to mobile phones and pagers		*/
/*									*/
/* vodafone.c								*/
/*									*/
/*  Copyright (C) 1997,1998 Angelo Masci				*/
/*									*/
/*  This library is free software; you can redistribute it and/or	*/
/*  modify it under the terms of the GNU Library General Public		*/
/*  License as published by the Free Software Foundation; either	*/
/*  version 2 of the License, or (at your option) any later version.	*/
/*									*/
/*  This library is distributed in the hope that it will be useful,	*/
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU	*/
/*  Library General Public License for more details.			*/
/*									*/
/*  You should have received a copy of the GNU Library General Public	*/
/*  License along with this library; if not, write to the Free		*/
/*  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.	*/
/*									*/
/*  You can contact the author at this e-mail address:			*/
/*									*/
/*  angelo@styx.demon.co.uk						*/
/*									*/
/* -------------------------------------------------------------------- */
/* $Id: vodafone.c,v 4.3 1998/01/17 22:49:15 root Exp $
   -------------------------------------------------------------------- */

#include <stdio.h>
#include <termios.h>
#include <strings.h>

#include <dial/modems.h>

#include "logfile.h"
#include "driver.h"
#include "expect.h"
#include "sms_modem.h"
#include "sms_error.h"
#include "sms_list.h"
#include "parserc.h"

/* -------------------------------------------------------------------- */

static char ACK1[] = "WELCOME TO THE VODAFONE TELENOTE SERVICE.\r\n";
static char ACK2[] = "PLEASE ENTER THE MOBILE NUMBER IN INTERNATIONAL FORMAT, E.G. 44385XXXXXX,\r\n"
	             "FOLLOWED BY RETURN, (OR RETURN TO QUIT).\r\n"
	             "\r\n"
	             ">";
	             
static char ACK3[] = "PLEASE TYPE YOUR TELENOTE, (MAXIMUM 160 CHARACTERS), FOLLOWED BY RETURN.\r\n"
		     "\r\n"
		     ">";

static char ACK4[] = "MESSAGE ACCEPTED.\r\n";
		     
/* -------------------------------------------------------------------- */

static int 	fd,
		fd_in,
		fd_out;

		
#define LONGTIMEOUT 	30
#define MAX_BUFSIZE	1024
#define TIMEOUT 	5

/* -------------------------------------------------------------------- */

static int VODAFONE_dial(char *number, char *service);
static void VODAFONE_hangup(void);
static int VODAFONE_sendmessage(char *msisdn, char *message);
	
/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int VODAFONE_dial(char *number, char *service)
{
	lprintf(LOG_STANDARD, "Dialing %s SMSC %s...\n", service, number);

	fd = SMS_dial(number, service);
	if (fd < 0)
	{
		mdmperror("Dialing");
		return EDIAL;
	}
	
	fd_in  = fd;
	fd_out = fd;

	lprintf(LOG_STANDARD, "Connection Established.\n");

	return 0;
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static void VODAFONE_hangup(void)
{
	lprintf(LOG_STANDARD, "Hangup...\n");	
	hangup(fd);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
static int VODAFONE_sendmessage(char *msisdn, char *message)
{
	char buf[1024];
	char nmessage[1024];


	if (expstr(fd_in, buf, ACK1, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Vodafone Service Login\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Vodafone Service Response\n");
		
		VODAFONE_hangup();
		return EVODAFONE_NORESPONSE;
	}



	if (expstr(fd_in, buf, ACK2, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Number Request\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Number Request\n");
		
		VODAFONE_hangup();
		return EVODAFONE_NONUMBER;
	}



	twrite(fd_out, msisdn, strlen(msisdn), TIMEOUT);
	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);



	if (expstr(fd_in, buf, ACK3, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Request\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Message Request\n");
		
		VODAFONE_hangup();
		return EVODAFONE_NOMESSAGE;
	}



	strcpy(nmessage, message);
	strcat(nmessage, "\r\n");

	twrite(fd_out, nmessage, strlen(nmessage), TIMEOUT);

	
	if (expstr(fd_in, buf, ACK4, MAX_BUFSIZE, TIMEOUT) == 0)
	{
		lprintf(LOG_STANDARD, "Received Message Delivery Response\n");
	}
	else
	{
		lprintf(LOG_STANDARD, "No Message Delivery Response\n");
		
		VODAFONE_hangup();
		return EVODAFONE_NODELIVERY;
	}


	twrite(fd_out, "\r\n", strlen("\r\n"), TIMEOUT);

	return 0;
}

/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
void vodafone_deliver(SMS_list *node, char *message)
{
	char 	*msisdn, 
		*smsc_number,
		*name, 
		*mservice;

	int 	error;
	

	msisdn      = get_number(node);
	name        = get_name(node);
	mservice    = get_service(node);
	smsc_number = get_smscnumber(mservice);

	lprintf(LOG_VERBOSE, "Name     :%s\n", name);
	lprintf(LOG_VERBOSE, "Number   :%s\n", msisdn);


 	error = VODAFONE_dial(smsc_number, mservice);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	error = VODAFONE_sendmessage(msisdn, message);
	if (error)
	{
		set_delivery(node, error);
		return;
	}

	VODAFONE_hangup();
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
SMS_list *vodafone_main(SMS_list *list, char *message)
{
	SMS_list 
		*node;

	lprintf(LOG_VERBOSE, "%s driver called\n", vodafone_device.name);
	
	node = get_first(list);
	while (node != NULL)
	{
		set_delivery(node, 0);
		vodafone_deliver(node, message);
		node = get_next(node);
	}

	return NULL; 
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
int vodafone_validate_id(char *id)
{
	return is_numeric(id);
}


/* -------------------------------------------------------------------- */
/* -------------------------------------------------------------------- */
DEVICE_ENTRY vodafone_device = {
	"VODAFONE",
	vodafone_main,
	vodafone_validate_id
};


